package com.exh.bevel;

import java.util.LinkedList;
import java.util.List;

import android.graphics.RectF;

import com.exh.math.primitive.Geo;
import com.exh.math.primitive.Point3;

/* A collection of hardcoded point lists, one of which
 * is used as the initial input to the algorithm.
 */
public class Input {

	public static final List<Point3> initSilo() {
		final List<Point3>	pts = new LinkedList<Point3>();
		pts.add(new Point3(130, 80, 0));
		pts.add(new Point3(230, 80, 0));
		pts.add(new Point3(280, 130, 0));
		pts.add(new Point3(280, 280, 0));
		pts.add(new Point3(230, 280, 0));
		pts.add(new Point3(230, 230, 0));
		pts.add(new Point3(180, 170, 0));
		pts.add(new Point3(130, 230, 0));
		pts.add(new Point3(130, 280, 0));
		pts.add(new Point3(80, 280, 0));
		pts.add(new Point3(80, 130, 0));
		return pts;
	}

	public static final List<Point3> initFlip() {
		final List<Point3>	pts = new LinkedList<Point3>();
		pts.add(new Point3(0, 0, 0));
		pts.add(new Point3(200, 0, 0));
		pts.add(new Point3(200, 200, 0));
		pts.add(new Point3(100, 200, 0));
		pts.add(new Point3(60, 190, 0));
		pts.add(new Point3(50, 170, 0));
		pts.add(new Point3(40, 130, 0));
		pts.add(new Point3(0, 100, 0));
		return pts;
	}

	public static final List<Point3> initSquare() {
		final List<Point3>	pts = new LinkedList<Point3>();
		pts.add(new Point3(80, 80, 0));
		pts.add(new Point3(280, 80, 0));
		pts.add(new Point3(280, 280, 0));
		pts.add(new Point3(80, 280, 0));
		return pts;
	}

	public static final List<Point3> initQuad() {
		final List<Point3>	pts = new LinkedList<Point3>();
		pts.add(new Point3(80, 75, 0));
		pts.add(new Point3(280, 80, 0));
		pts.add(new Point3(285, 280, 0));
		pts.add(new Point3(75, 280, 0));
		return pts;
	}

	public static final List<Point3> initT() {
		final List<Point3>	pts = new LinkedList<Point3>();
		pts.add(new Point3(80, 80, 0));
		pts.add(new Point3(280, 80, 0));
		pts.add(new Point3(280, 180, 0));
		pts.add(new Point3(230, 180, 0));
		pts.add(new Point3(230, 280, 0));
		pts.add(new Point3(130, 280, 0));
		pts.add(new Point3(130, 180, 0));
		pts.add(new Point3(80, 180, 0));
		return pts;
	}

	public static final List<Point3> initCircle(final int steps, final double rotate) {
		final List<Point3>	pts = new LinkedList<Point3>();
		final RectF		b = new RectF(0, 0, 256, 256);
		final Point3	cen = new Point3(b.centerX(), b.centerY(), 0);
		final double	dist = cen.x - b.left;
		final double	scale = (360.0 / (double)(steps));

		for (int k=0; k<steps; k++) {
			final Point3	pt = new Point3(0, 0, 0);
			double			deg = rotate + (k*scale);
			if (deg >= 360) deg -= 360;
			getCircleEdge(cen,360-deg, dist, pt);
			pts.add(pt);
		}
		return pts;
	}

	// 0 = right, counterclockwise (90 = up)
	private final static void getCircleEdge(final Point3 cen, double degree, final double dist,
											final Point3 out) {
		degree *= Geo.DEGREES_TO_RADIANS;
		out.x = (cen.x + (Math.cos(degree) * dist));
		out.y = (cen.y - (Math.sin(degree) * dist));
	}

}
