package com.exh.bevel;

import java.util.List;

import com.exh.bevel.algo.Bevel;
import com.exh.bevel.algo.QuadClipper;
import com.exh.bevel.support.QuadList;
import com.exh.bevel.support.Spotlight;
import com.exh.math.primitive.Point3;

import android.graphics.Bitmap;
import android.graphics.Point;

/* The entry point for the algorithm.  Create the initial point list
 * input data, extrude it with the beveller, and convert it to a
 * bitmap with the renderer.
 */
public final class Silo implements Generator {
	
	private final Point				mSize = new Point();
	
//	private final static float		MASTER_DEPTH = 128;
//	private final static float		MASTER_DEPTH = 125;
	private final static float		MASTER_DEPTH = 90;
//	private final static float		MASTER_DEPTH = 60;

//	private final static float		MASTER_DEPTH = 40;

//	private final static float		MASTER_DEPTH = 30;
	private final static double		BEVEL_AMOUNT = MASTER_DEPTH;
	
//	private final static int		RENDER_F = (Render.FILL | Render.STROKE);
	private final static int		RENDER_F = (Render.RENDER);
	
	// This determines the final Z value, which is scaled
	// according to the bevel amount.  Higher values mean
	// taller peaks, which means more pronounced lighting.
	private final static double		HEIGHT_SCALE = 0.75;
	
	/* SILO
	 ********/
	public Silo() {
	}
	
	public Bitmap run(final ProgressReport progress) {
		// OK, I don't have to JUST create the silo shape.
//		final List<Point3>			pts = Input.initQuad();
//		final List<Point3>			pts = Input.initT();
//		final List<Point3>			pts = Input.initCircle(5, 270);	// pentagon
//		final List<Point3>			pts = Input.initCircle(20, 0);
		final List<Point3>			pts = Input.initSilo();
		
		return run(pts, BEVEL_AMOUNT, RENDER_F, progress);
	}
	
	public Bitmap run(	final List<Point3> input, final double amount,
						final int flags, final ProgressReport progress) {
		progress.setProgressReport(0);
		
		translateToBounds(input, mSize);

		try {
			// On object used to highlight geometry, for error reporting,
			// and general poking into what's going on.
			final Spotlight			spot = new Spotlight();
			final double			height = amount * HEIGHT_SCALE;
			
			// Take the list of points and extrude them into 3d quads.
			final QuadList			extruded = new QuadList();
			final Bevel				bevel = new Bevel();
			bevel.asBevel(input, amount, 1, extruded);

			// Take the extruded quads and clip any overlaps.			
			final QuadList			clipped = new QuadList();
			final QuadClipper		clipper = new QuadClipper();
			if ((flags&Render.NO_CLIP) != 0) {
				// If clipping is off, just place all my quads in the result list
				extruded.swap(clipped);
			} else {
				clipper.on(extruded, height, spot, clipped);
			}
			
			// Render the clipped quads.
			final Render			render = new Render();
			return render.newImage(clipped, mSize.x, mSize.y, flags, height, spot, progress);
		} catch (Exception ex) {
			System.out.println("ERROR in root=" + ex);
		}
		return null;
	}

	// Given a list of points, make them fit inside the standard bounds.
	final static void translateToBounds(final List<Point3> pts, final Point outSize) {
		final float					BORDER = 4;
		final int					W = 256, H = 256;
		translateToBounds(pts, BORDER, BORDER, W-BORDER, H-BORDER);
		if (outSize != null) {
			outSize.x = W;
			outSize.y = H;
		}
	}

	// Given a list of points, make them fit inside the given bounds.
	final static void translateToBounds(final List<Point3> pts,
										final double left, final double top,
										final double right, final double bottom) {
		double			minX = pts.get(0).x, minY = pts.get(0).y;
		double			maxX = minX, maxY = minY;
		for (Point3 pt : pts) {
			if (pt.x < minX) minX = pt.x;
			else if (pt.x > maxX) maxX = pt.x;
			if (pt.y < minY) minY = pt.y;
			else if (pt.y > maxY) maxY = pt.y;
		}
		final double	scaleX = (right-left)/(maxX-minX),
						scaleY = (bottom-top)/(maxY-minY);
		for (Point3 pt : pts) {
			pt.x = left + ((pt.x-minX)*scaleX);
			pt.y = top + ((pt.y-minY)*scaleY);
		}
	}
}