package com.exh.bevel.support;

import android.graphics.RectF;

import com.exh.math.primitive.Point3;
import com.exh.math.primitive.Quad;

public final class QuadEntry {
	
	public QuadEntry		mNext = null;
	public final Quad		mQ = new Quad();
	
	// For faster intersection tests
	private final RectF		mFrame = new RectF();

	public QuadEntry() {
	}
	
	public void setQuad(final Quad q) {
		mQ.setTo(q);
		cacheFrame();
	}
	
	public void setQuad(final Point3 a, final Point3 b, final Point3 c, final Point3 d) {
		mQ.mPt[0].set(a);
		mQ.mPt[1].set(b);
		mQ.mPt[2].set(c);
		mQ.mPt[3].set(d);
		cacheFrame();
	}
	
	public void insert(final QuadEntry o) {
		o.tail().mNext = mNext;
		mNext = o;
	}
	
	// A broad check to see if we might intersect
	public boolean overlaps(final QuadEntry o) {
		// Don't process myself
		if (this == o) return false;
		// Don't process neighbors
		if (mQ.mPt[0].similarTo(o.mQ.mPt[3]) || mQ.mPt[3].similarTo(o.mQ.mPt[0])) return false;
		// If our frames overlap in 2d space then we MIGHT intersect.
		return RectF.intersects(mFrame, o.mFrame);
	}
	
	public QuadEntry tail() {
		QuadEntry			e = this;
		while (e.mNext != null) e = e.mNext;
		return e;
	}

	private void cacheFrame() {
		mFrame.left = mFrame.right = (float)mQ.mPt[0].x;
		mFrame.top = mFrame.bottom = (float)mQ.mPt[0].y;
		for (int k=1; k<mQ.mPt.length; k++) {
			mFrame.left = Math.min(mFrame.left, (float)mQ.mPt[k].x);
			mFrame.top = Math.min(mFrame.top, (float)mQ.mPt[k].y);
			mFrame.right = Math.max(mFrame.right, (float)mQ.mPt[k].x);
			mFrame.bottom = Math.max(mFrame.bottom, (float)mQ.mPt[k].y);
		}
	}
}
