package com.exh.bevel.support;

import java.util.Iterator;
import java.util.NoSuchElementException;

import com.exh.math.primitive.Quad;

public class QuadList implements Iterable<QuadEntry> {

	private QuadEntry		mHead = null;

	public Iterator<QuadEntry> iterator() {
		return new EIterator(mHead);
	}
	
	public QuadEntry head() {
		return mHead;
	}
	
	public QuadEntry tail() {
		if (mHead != null) return mHead.tail();
		return null;
	}
	
	public void swap(final QuadList o) {
		final QuadEntry		e = mHead;		mHead = o.mHead;	o.mHead = e;
	}
	
	public void push(final QuadEntry e) {
		e.mNext = mHead;
		mHead = e;
	}
	
	public QuadEntry pop() {
		if (mHead == null) return new QuadEntry();
		final QuadEntry		ans = mHead;
		mHead = mHead.mNext;
		ans.mNext = null;
		return ans;
	}
	
	public QuadEntry pop(final Quad q) {
		final QuadEntry		ans = pop();
		ans.setQuad(q);
		return ans;
	}
	
	public QuadEntry pop(final QuadEntry e) {
		final QuadEntry		ans = pop();
		ans.setQuad(e.mQ);
		return ans;
	}

	public QuadEntry popAll() {
		final QuadEntry		ans = mHead;
		mHead = null;
		return ans;
	}
	
	public void moveTo(final QuadList o) {
		if (mHead != null) {
			if (o.mHead == null) o.mHead = mHead;
			else o.mHead.tail().mNext = mHead;
			mHead = null;
		}
	}
	
	/* E-ITERATOR
	 */
	private final static class EIterator implements Iterator<QuadEntry> {
		
		private QuadEntry		mE = null;
		
		EIterator(final QuadEntry e) {
			mE = e;
		}
		
		public boolean hasNext() {
			return mE != null;
		}
		
		public QuadEntry next() {
			if (mE == null) throw new NoSuchElementException();
			final QuadEntry		ans = mE;
			mE = mE.mNext;
			return ans;
		}
		
		public void remove() {
			throw new UnsupportedOperationException();
		}
	}
}
