package com.exh.bevel.support;

import com.exh.math.RayIntersectsQuad;
import com.exh.math.RayIntersectsTri;
import com.exh.math.primitive.Point3;
import com.exh.math.primitive.Quad;
import com.exh.math.primitive.Segment;

/* A special intersection test, since I've chosen to operate on a
 * quad-based format that allows degenerate quads (forming triangles).
 */
public class RayIntersectsSurface {

	private final RayIntersectsQuad		mRIQ = new RayIntersectsQuad();
	private final RayIntersectsTri		mRIT = new RayIntersectsTri();

	public int on(final Segment ray, final Quad q, final Point3 out) {
		int				ans = RayIntersectsQuad.ERROR;
		if (isInsideTri(q)) {
			ans = mRIT.on(ray, q.mPt[0], q.mPt[1], q.mPt[3], out);
			if (ans == RayIntersectsTri.INTERSECT_TRI) ans = RayIntersectsQuad.INTERSECT_QUAD;
		} else if (isOutsideTri(q)) {
			ans = mRIT.on(ray, q.mPt[0], q.mPt[1], q.mPt[2], out);
			if (ans == RayIntersectsTri.INTERSECT_TRI) ans = RayIntersectsQuad.INTERSECT_QUAD;
		} else {
			ans = mRIQ.on(ray, q, out);
		}
		return ans;
	}

	// My data model lets quads collapse into tris on the inside.
	private final static boolean isInsideTri(final Quad q) {
		return q.mPt[1].sameAs(q.mPt[2]);
	}

	// My data model lets quads collapse into tris on the outside.
	private final static boolean isOutsideTri(final Quad q) {
		return q.mPt[0].sameAs(q.mPt[3]);
	}

}
