package com.exh.math;

import com.exh.math.primitive.Point3;
import com.exh.math.primitive.Quad;
import com.exh.math.primitive.Segment;

/* Class to determine if and where a ray interesects a quad.
 */
public final class RayIntersectsQuad {
	
	// When the function is done, this will answer whether or
	// not the actual segment intersected (provided the test succeeded).
	public boolean					mSegmentIntersects = false;

	public final static int			ERROR = -1;
	public final static int			NO_INTERSECT = 0;
	public final static int			INSIDE = 1;
	// The following are actual success conditions, and if
	// they're true, then out is valid
	public final static int			INTERSECT_PLANE = 2;
	public final static int			INTERSECT_QUAD = 3;

	private final RayIntersectsTri	mRIT = new RayIntersectsTri();
	
	/* Answer a value determining the type of intersection.  Out
	 * is valid on an answer of INTERSECT_PLANE or INTERSECT_QUAD.
	 * Quad is CCW.  This is just built from two RayIntersectsTri
	 * tests, but a little more efficient then actually doing two.
	 */
	public int on(final Segment ray, final Quad q, final Point3 out) {
		// Check the first tri in this quad
		int				ans = mRIT.on(ray, q.mPt[0], q.mPt[1], q.mPt[2], out);
		mSegmentIntersects = mRIT.mSegmentIntersects;
		if (ans < RayIntersectsTri.INTERSECT_PLANE) return ans;
		if (ans == RayIntersectsTri.INTERSECT_TRI) return INTERSECT_QUAD;
		// Check the second tri
		ans = mRIT.on(ray, q.mPt[0], q.mPt[2], q.mPt[3], out);
		mSegmentIntersects = mRIT.mSegmentIntersects;
		if (ans == RayIntersectsTri.INTERSECT_TRI) {
			return INTERSECT_QUAD;
		}
		return INTERSECT_PLANE;
	}
}
