package com.exh.math;

import com.exh.math.primitive.Point3;
import com.exh.math.primitive.Segment;

// http://softsurfer.com/Archive/algorithm_0106/algorithm_0106.htm#dist3D_Segment_to_Segment()

public final class SegIntersectsSeg {

	public double					mDistanceToHit;
	public final Point3				mHit = new Point3();
	
	private final Point3			mU = new Point3(),
									mV = new Point3(),
									mW = new Point3(),
									mY = new Point3(),
									mDp = new Point3();

	private final static double		EPS = 0.0000000001;

	// Given two segments, answer true if they intersect, otherwise false.
	// If true, mDistanceToHit and mHit will be valid.
	public boolean on(final Segment S1, final Segment S2) {
		mU.sub(S1.mPt[1], S1.mPt[0]);
		mV.sub(S2.mPt[1], S2.mPt[0]);
		mW.sub(S1.mPt[0], S2.mPt[0]);
		final double	a = mU.dot3(mU),
						b = mU.dot3(mV),
						c = mV.dot3(mV),
						d = mU.dot3(mW),
						e = mV.dot3(mW);
		final double	D = a*c - b*b;
		double			sc, sN, sD = D,
						tc, tN, tD = D;

		// compute the line parameters of the two closest points
		if (D < EPS) {			// the lines are almost parallel
			sN = 0.0;			// force using point P0 on segment S1
			sD = 1.0;			// to prevent possible division by 0.0 later
			tN = e;
			tD = c;
		} else {                // get the closest points on the infinite lines
			sN = (b*e - c*d);
			tN = (a*e - b*d);
			if (sN < 0.0) {       // sc < 0 => the s=0 edge is visible
				sN = 0.0;
				tN = e;
				tD = c;
			} else if (sN > sD) {  // sc > 1 => the s=1 edge is visible
				sN = sD;
				tN = e + b;
				tD = c;
			}
		}

		if (tN < 0.0) {           // tc < 0 => the t=0 edge is visible
			tN = 0.0;
			// recompute sc for this edge
			if (-d < 0.0) sN = 0.0;
			else if (-d > a) sN = sD;
			else {
				sN = -d;
				sD = a;
			}
		} else if (tN > tD) {      // tc > 1 => the t=1 edge is visible
			tN = tD;
			// recompute sc for this edge
			if ((-d + b) < 0.0) sN = 0;
			else if ((-d + b) > a) sN = sD;
			else {
				sN = (-d + b);
				sD = a;
			}
		}
		// finally do the division to get sc and tc
		sc = (Math.abs(sN) < EPS ? 0.0 : sN / sD);
		tc = (Math.abs(tN) < EPS ? 0.0 : tN / tD);

		// The hit point
		mHit.sub(S1.mPt[1], mU);
//		mPt1.sub(S2.mPt[1], mV);

	    // get the difference of the two closest points
	    mU.mult(sc);
	    mV.mult(tc);
	    mY.sub(mU, mV);
	    mDp.add(mW, mY);
//	    Vector   dP = w + (sc * u) - (tc * v);  // = S1(sc) - S2(tc)

		// return the closest distance
		mDistanceToHit = mDp.length();
		return mDistanceToHit < EPS;
	}
}