package com.exh.math.primitive;

import android.graphics.Path;

/* I always store my own point data.
 */
public class Polygon {

	public Point3[]			mPt = null;
	public int				mSize = 0;

	public void clear() {
		mSize = 0;
	}
	
	public Polygon copy() {
		final Polygon		ans = new Polygon();
		if (mSize < 1) return ans;
		ans.mPt = new Point3[mSize];
		ans.mSize = mSize;
		for (int k=0; k<mSize; k++) ans.mPt[k] = new Point3(mPt[k].x, mPt[k].y, mPt[k].z);
		return ans;
	}
	
	public void swap(final Polygon o) {
		final Point3[]		pt = mPt;	mPt = o.mPt;		o.mPt = pt;
		final int			s = mSize;	mSize = o.mSize;	o.mSize = s;
	}
	
	public void setAlloc(final int newAlloc) {
		if (newAlloc <= 0) {
			mSize = 0;
			return;
		}
		if (mPt == null || mPt.length < newAlloc) {
			final Point3[]		newPt = new Point3[newAlloc];
			if (mPt != null) System.arraycopy(mPt, 0, newPt, 0, mPt.length);
			mPt = newPt;
		}
	}
	
	public void rewind() {
		mSize = 0;
	}
	
	public void add(final Point3 pt) {
		add(pt.x, pt.y, pt.z);
	}
	
	public void add(final double x, final double y, final double z) {
		if (mPt == null || mSize >= mPt.length) setAlloc(mSize+4);
		
		if (mPt[mSize] == null) mPt[mSize] = new Point3();
		mPt[mSize].set(x, y, z);
		mSize++;
	}
	
	public void remove(final int idx) {
		if (idx < 0 || idx >= mSize) return;

		for (int k=idx; k<mSize-1; k++) {
			mPt[k].set(mPt[k+1]);
		}
		mSize--;
	}
	
	public void setTo(final Point3[] pts) {
		setTo(pts, pts.length);
	}
	
	public void setTo(final Point3[] pts, final int size) {
		mSize = 0;
		for (int k=0; k<size; k++) add(pts[k]);
	}

	// Drawing/testing
	public void addToPath(final Path p) {
		if (mSize < 2) return;
		
		Point3			pt = mPt[mSize-1];
		p.moveTo((float)pt.x, (float)pt.y);
		for (int k=0; k<mSize; k++) {
			pt = mPt[k];
			p.lineTo((float)pt.x, (float)pt.y);
		}
	}
	
	public void print() {
		System.out.println("Polygon size=" + mSize);
		for (int k=0; k<mSize; k++) System.out.println("\t" + k + "=" + mPt[k]);
	}

}
