package com.exh.math.primitive;

// Quadrilateral
public final class Quad {
	public final static int				SIZE = 4;

	// The client treats the quad as counterclockwise, starting with
	// the first outer point.  That means segment 0, 3 is the outside
	// edges and 1, 2 is the inside.
	public final Point3[]				mPt = Point3.newArray(SIZE);
	
	public final static Quad			create(final Point3 p0, final Point3 p1, final Point3 p2, final Point3 p3) {
		final Quad		o = new Quad();
		o.mPt[0].set(p0);
		o.mPt[1].set(p1);
		o.mPt[2].set(p2);
		o.mPt[3].set(p3);
		return o;
	}
	
	public void setTo(final Quad o) {
		for (int k=0; k<mPt.length; k++) mPt[k].set(o.mPt[k]);
	}
	
	public void set(final Point3 a, final Point3 b, final Point3 c, final Point3 d) {
		mPt[0].set(a);
		mPt[1].set(b);
		mPt[2].set(c);
		mPt[3].set(d);
	}

	public void appendTo(final Polygon p) {
		for (int k=0; k<mPt.length; k++) p.add(mPt[k]);
	}

	public Polygon asPolygon() {
		final Polygon		ans = new Polygon();
		appendTo(ans);
		return ans;
	}

	public boolean containsCCW(final double x, final double y) {
		// Special rule for my degenerate quads that look like tris.
		if (mPt[1].sameAs(mPt[2])) return isInside(mPt[0].x, mPt[0].y, mPt[1].x, mPt[1].y, mPt[3].x, mPt[3].y, x, y);
		
		if (isInside(mPt[0].x, mPt[0].y, mPt[1].x, mPt[1].y, mPt[2].x, mPt[2].y, x, y)) return true;
		return isInside(mPt[0].x, mPt[0].y, mPt[2].x, mPt[2].y, mPt[3].x, mPt[3].y, x, y);
	}

	public final double area() {
		return getArea(mPt[0].x, mPt[0].y, mPt[3].x, mPt[3].y, mPt[1].x, mPt[1].y)
				+ getArea(mPt[0].x, mPt[0].y, mPt[3].x, mPt[3].y, mPt[2].x, mPt[2].y);
	}
	
	// Technically a function on a triangle, if we had one.
	// This works by taking the given point, and converting the
	// triangle into three triangles with the new point, then
	// comparing the area of that with the area of the triangle.
	final static boolean isInside(	final double ax, final double ay,
									final double bx, final double by,
									final double cx, final double cy,
									final double x, final double y) {
		final double		t = getArea(ax, ay, bx, by, cx, cy);
		double				sum = getArea(ax, ay, bx, by, x, y);
		if (sum > t) return false;
	
		sum += getArea(bx, by, cx, cy, x, y);
		if (sum > t) return false;

		sum += getArea(ax, ay, cx, cy, x, y);
		// Oh man...  there's some sort of small error
		// to account for.  Where does that come from?
		return sum-0.00000001 <= t;
	}

	private final static double getArea(final double ax, final double ay,
										final double bx, final double by,
										final double cx, final double cy) {
		final double		a = (ax - cx),
					 		b = (ay - cy),
					 		c = (bx - cx),
					 		d = (by - cy);
		return 0.5 * Math.abs((a*d) - (b*c));
	}
	
	public String toString() {
		return mPt[0] + " " + mPt[1] + " " + mPt[2] + " " + mPt[3];
	}
}
